﻿using ComputerShopContracts.BindingModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using ComputerShopDataModels.Enums;
using ComputerShopContracts.ViewModels;
using ComputerShopDataModels.Models;

namespace ComputerShopDatabaseImplement.Models
{
    internal class Order : IOrderModel
    {
        public int Id { get; private set; }
        [Required]
        public double Sum { get; private set; }
        [Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;
        [Required]
        public DateTime DateCreate { get; private set; } = DateTime.Now;

        public DateTime? DateImplement { get; private set; }
        [ForeignKey("OrderId")]
        public virtual List<SupplyOrder> SupplyOrders { get; set; } = new();

        [ForeignKey("OrderId")]
        public virtual List<AssemblyOrder> Assemblies { get; set; } = new();
        private Dictionary<int, (IOrderModel, int)>? _assemblyOrders = null;

        [NotMapped]
        public Dictionary<int, (IOrderModel, int)> AssemblyOrders
        {
            get
            {
                if (_assemblyOrders == null)
                {
                    _assemblyOrders = Assemblies
                    .ToDictionary(recPC => recPC.AssemblyId, recPC =>
                    (recPC.Order as IOrderModel, recPC.Count));
                }
                return _assemblyOrders;
            }
        }

        [Required]
        public int ClientId { get; set; }
        public static Order? Create(OrderBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                ClientId = model.ClientId
            };
        }
        public static Order Create(OrderViewModel model)
        {
            return new Order
            {
                Id = model.Id,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                ClientId = model.ClientId
            };
        }
        public void Update(OrderBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            ClientId = ClientId
        };
    }
}
